@@ -19,6 +19,7 @@
 #include "mutt.h"
 #include "mutt_curses.h"
 #include "mutt_menu.h"
+#include "mbyte.h"
 
 #ifdef USE_IMAP
 #include "imap.h"
@@ -27,8 +28,16 @@
 #include <string.h>
 #include <stdlib.h>
 
+extern int Charset_is_utf8; /* FIXME: bad modularisation */
+
 static void print_enriched_string (int attr, unsigned char *s, int do_color)
 {
+  wchar_t wc;
+  size_t k;
+  size_t n = mutt_strlen ((char *)s);
+  mbstate_t mbstate;
+
+  memset (&mbstate, 0, sizeof (mbstate));
   while (*s)
   {
     if (*s < M_TREE_MAX)
@@ -40,19 +49,60 @@ static void print_enriched_string (int attr, unsigned char *s, int do_color)
 	switch (*s)
 	{
 	  case M_TREE_LLCORNER:
-	    addch (option (OPTASCIICHARS) ? '`' : ACS_LLCORNER);
+	    if (option (OPTASCIICHARS))
+	      addch ('`');
+	    else if (Charset_is_utf8)
+	      addstr ("\342\224\224"); /* WACS_LLCORNER */
+	    else
+	      addch (ACS_LLCORNER);
 	    break;
 	  case M_TREE_ULCORNER:
-	    addch (option (OPTASCIICHARS) ? ',' : ACS_ULCORNER);
+	    if (option (OPTASCIICHARS))
+	      addch (',');
+	    else if (Charset_is_utf8)
+	      addstr ("\342\224\214"); /* WACS_ULCORNER */
+	    else
+	      addch (ACS_ULCORNER);
 	    break;
 	  case M_TREE_LTEE:
-	    addch (option (OPTASCIICHARS) ? '|' : ACS_LTEE);
+	    if (option (OPTASCIICHARS))
+	      addch ('|');
+	    else if (Charset_is_utf8)
+	      addstr ("\342\224\234"); /* WACS_LTEE */
+	    else
+	      addch (ACS_LTEE);
 	    break;
 	  case M_TREE_HLINE:
-	    addch (option (OPTASCIICHARS) ? '-' : ACS_HLINE);
+	    if (option (OPTASCIICHARS))
+	      addch ('-');
+	    else if (Charset_is_utf8)
+	      addstr ("\342\224\200"); /* WACS_HLINE */
+	    else
+	      addch (ACS_HLINE);
 	    break;
 	  case M_TREE_VLINE:
-	    addch (option (OPTASCIICHARS) ? '|' : ACS_VLINE);
+	    if (option (OPTASCIICHARS))
+	      addch ('|');
+	    else if (Charset_is_utf8)
+	      addstr ("\342\224\202"); /* WACS_VLINE */
+	    else
+	      addch (ACS_VLINE);
+	    break;
+	  case M_TREE_TTEE:
+	    if (option (OPTASCIICHARS))
+	      addch ('-');
+	    else if (Charset_is_utf8)
+	      addstr ("\342\224\254"); /* WACS_TTEE */
+	    else
+	      addch (ACS_TTEE);
+	    break;
+	  case M_TREE_BTEE:
+	    if (option (OPTASCIICHARS))
+	      addch ('-');
+	    else if (Charset_is_utf8)
+	      addstr ("\342\224\264"); /* WACS_BTEE */
+	    else
+	      addch (ACS_BTEE);
 	    break;
 	  case M_TREE_SPACE:
 	    addch (' ');
@@ -66,16 +116,24 @@ static void print_enriched_string (int attr, unsigned char *s, int do_color)
 	  case M_TREE_HIDDEN:
 	    addch ('&');
 	    break;
+	  case M_TREE_EQUALS:
+	    addch ('=');
+	    break;
+	  case M_TREE_MISSING:
+	    addch ('?');
+	    break;
 	}
-	s++;
+	s++, n--;
       }
       if (do_color) attrset(attr);
     }
-    else
+    else if ((k = mbrtowc (&wc, (char *)s, n, &mbstate)) > 0)
     {
-      addch (*s);
-      s++;
+      addnstr ((char *)s, k);
+      s += k, n-= k;
     }
+    else
+      break;
   }
 }
 
@@ -90,27 +148,13 @@ static void menu_make_entry (char *s, int l, MUTTMENU *menu, int i)
     menu->make_entry (s, l, menu, i);
 }
 
-void menu_pad_string (char *s, size_t l)
+void menu_pad_string (char *s, size_t n)
 {
-#if !defined(HAVE_BKGDSET) && !defined (USE_SLANG_CURSES)
-  int n = mutt_strlen (s);
-#endif
   int shift = option (OPTARROWCURSOR) ? 3 : 0;
-  
-  l--; /* save room for the terminal \0 */
-  if (l > COLS - shift)
-    l = COLS - shift;
-#if !defined (HAVE_BKGDSET) && !defined (USE_SLANG_CURSES)
-  /* we have to pad the string with blanks to the end of line */
-  if (n < l)
-  {
-    while (n < l)
-      s[n++] = ' ';
-    s[n] = 0;
-  }
-  else
-#endif
-    s[l] = 0;
+  int cols = COLS - shift;
+
+  mutt_format_string (s, n, cols, cols, 0, ' ', s, strlen (s), 1);
+  s[n - 1] = 0;
 }
 
 void menu_redraw_full (MUTTMENU *menu)
@@ -123,7 +167,8 @@ void menu_redraw_full (MUTTMENU *menu)
   if (option (OPTHELP))
   {
     SETCOLOR (MT_COLOR_STATUS);
-    mvprintw (option (OPTSTATUSONTOP) ? LINES-2 : 0, 0, "%-*.*s", COLS, COLS, menu->help);
+    move (option (OPTSTATUSONTOP) ? LINES-2 : 0, 0);
+    mutt_paddstr (COLS, menu->help);
     SETCOLOR (MT_COLOR_NORMAL);
     menu->offset = 1;
     menu->pagelen = LINES - 3;
@@ -145,7 +190,8 @@ void menu_redraw_status (MUTTMENU *menu)
 
   snprintf (buf, sizeof (buf), M_MODEFMT, menu->title);
   SETCOLOR (MT_COLOR_STATUS);
-  mvprintw (option (OPTSTATUSONTOP) ? 0 : LINES - 2, 0, "%-*.*s", COLS, COLS, buf);
+  move (option (OPTSTATUSONTOP) ? 0 : LINES - 2, 0);
+  mutt_paddstr (COLS, buf);
   SETCOLOR (MT_COLOR_NORMAL);
   menu->redraw &= ~REDRAW_STATUS;
 }
@@ -299,7 +345,7 @@ void menu_redraw_prompt (MUTTMENU *menu)
   {
     if (option (OPTMSGERR)) 
     {
-      sleep (1);
+      mutt_sleep (1);
       unset_option (OPTMSGERR);
     }
 
@@ -322,7 +368,7 @@ void menu_check_recenter (MUTTMENU *menu)
   }
   else if (menu->current >= menu->top + menu->pagelen)
   {
-    if (option (OPTMENUSCROLL))
+    if (option (OPTMENUSCROLL) || (menu->pagelen <= 0))
       menu->top = menu->current - menu->pagelen + 1;
     else
       menu->top += menu->pagelen * ((menu->current - menu->top) / menu->pagelen);
@@ -330,7 +376,7 @@ void menu_check_recenter (MUTTMENU *menu)
   }
   else if (menu->current < menu->top)
   {
-    if (option (OPTMENUSCROLL))
+    if (option (OPTMENUSCROLL) || (menu->pagelen <= 0))
       menu->top = menu->current;
     else
     {
@@ -753,6 +799,13 @@ int mutt_menuLoop (MUTTMENU *menu)
 
   FOREVER
   {
+    if (option (OPTMENUCALLER))
+    {
+      unset_option (OPTMENUCALLER);
+      return OP_NULL;
+    }
+    
+    
     mutt_curs_set (0);
 
 #ifdef USE_IMAP
@@ -925,9 +978,15 @@ int mutt_menuLoop (MUTTMENU *menu)
       case OP_TAG:
 	if (menu->tag && !menu->dialog)
 	{
-	  if (menu->max)
+	  if (menu->tagprefix && !option (OPTAUTOTAG))
 	  {
-	    int i = menu->tag (menu, menu->current);
+	    for (i = 0; i < menu->max; i++)
+	      menu->tagged += menu->tag (menu, i, 0);
+	    menu->redraw = REDRAW_INDEX;
+	  }
+	  else if (menu->max)
+	  {
+	    int i = menu->tag (menu, menu->current, -1);
 	    menu->tagged += i;
 	    if (i && option (OPTRESOLVE) && menu->current < menu->max - 1)
 	    {
